// 着色器程序使用介绍：
// uniform变量的值由应用程序使用glUniform**方法传递给着色器程序，着色器程序只读，无法修改
// 如果uniform变量在vertex和fragment两者之间声明方式完全一样，则它可以在vertex和fragment共享使用。（相当于一个被vertex和fragment shader共享的全局变量）
// uniform变量一般用来表示：变换矩阵，材质，光照参数和颜色等信息；有个数限制，但至少支持128个顶点uniform和16个片段（Fragment）uniform
//
// attribute变量是只能在顶点着色器中使用的变量（片元着色器无法使用）
// 一般用attribute变量来表示一些顶点的数据，如：顶点坐标，法线，纹理坐标，顶点颜色等
// 在application中，一般用函数glBindAttribLocation（）来绑定每个attribute变量的位置，然后用函数glVertexAttribPointer（）为每个attribute变量赋值。
//
// varying变量是顶点着色器和片元着色器之间做数据传递用的。
// 一般顶点着色器修改varying变量的值，然后片元着色器使用该varying变量的值。因此varying变量在顶点着色器和片元着色器二者之间的声明必须是一致的。
// application不能使用此变量。

// 定义属性，代码中通过glVertexAttribPointer方法来给属性赋值
attribute vec4 aVertexCoordinates;  //顶点坐标
attribute vec4 aTextureCoordinates; //纹理坐标

// 顶点着色器和片元着色器传递变量
varying vec2 vTextureCoordinates; //转换后的纹理坐标，会传递给片元着色器

// 转换矩阵
uniform mat4 uTextureTransform; //用于纹理坐标转换

void main() {
    vTextureCoordinates = (uTextureTransform * aTextureCoordinates).xy; // 纹理坐标
    gl_Position = aVertexCoordinates; // 顶点坐标
}